#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-11-08
#
# Monitor Check Point Maestro SMO Chassis status (asg monitor)
#
# 2021-02-25: rewrite for CMK 2.x
#             added warning/critical is chassis grade not equeal maxGrade
# 2021-09-10: rewrite parse function
# 2021-09-11: added WATO, cleanup
# 2022-10-23: fixed warning on upgrade "non-empty params vanished" for discovery_chassis and discovery_sgms
#
# sample snmpwalk 
# .1.3.6.1.4.1.2620.1.48.28.1.0 = STRING: "Multi"
# .1.3.6.1.4.1.2620.1.48.28.2.0 = STRING: "Primary Up"
# .1.3.6.1.4.1.2620.1.48.28.3.1.0 = STRING: "Enabled"
# .1.3.6.1.4.1.2620.1.48.28.3.2.0 = STRING: "Enabled"
# .1.3.6.1.4.1.2620.1.48.28.4.1.1.1.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.48.28.4.1.1.2.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.1.48.28.4.1.2.1.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.48.28.4.1.2.2.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.1.48.28.4.1.3.1.0 = STRING: "ACTIVE"
# .1.3.6.1.4.1.2620.1.48.28.4.1.3.2.0 = STRING: "STANDBY"
# .1.3.6.1.4.1.2620.1.48.28.4.1.4.1.0 = STRING: "34"
# .1.3.6.1.4.1.2620.1.48.28.4.1.4.2.0 = STRING: "28"
# .1.3.6.1.4.1.2620.1.48.28.4.1.5.1.0 = STRING: "34"
# .1.3.6.1.4.1.2620.1.48.28.4.1.5.2.0 = STRING: "28"
# .1.3.6.1.4.1.2620.1.48.28.4.1.6.1.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.28.4.1.6.2.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.28.5.1.1.1.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.48.28.5.1.1.2.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.1.48.28.5.1.1.3.0 = Gauge32: 3
# .1.3.6.1.4.1.2620.1.48.28.5.1.2.1.0 = STRING: "1_01"
# .1.3.6.1.4.1.2620.1.48.28.5.1.2.2.0 = STRING: "1_02"
# .1.3.6.1.4.1.2620.1.48.28.5.1.2.3.0 = STRING: "2_01"
# .1.3.6.1.4.1.2620.1.48.28.5.1.3.1.0 = STRING: "ACTIVE"
# .1.3.6.1.4.1.2620.1.48.28.5.1.3.2.0 = STRING: "ACTIVE"
# .1.3.6.1.4.1.2620.1.48.28.5.1.3.3.0 = STRING: "ACTIVE"
# .1.3.6.1.4.1.2620.1.48.28.5.1.4.1.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.28.5.1.4.2.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.28.5.1.4.3.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.28.5.1.5.1.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.28.5.1.5.2.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.48.28.5.1.5.3.0 = STRING: "N/A"
#
# sample string_table
# [
#  [
#   ['Multi', 'Primary Up', 'Enabled', 'Enabled']
#  ],
#  [
#   ['1', 'ACTIVE', '34', '34', 'N/A'],
#   ['2', 'STANDBY', '28', '28', 'N/A']
#  ],
#  [
#   ['1_01', 'ACTIVE'],
#   ['1_02', 'ACTIVE'],
#   ['2_01', 'ACTIVE']
#  ]
# ]
#

from typing import List, NamedTuple, Optional

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    equals,
    Result,
    State,
    SNMPTree,
    startswith,
    all_of,
    any_of,
)


class CheckoointAsgChassisInfo(NamedTuple):
    mode: str
    hamode: str
    synctoactive: str
    synctostandby: str


class CheckpointAsgChassisParams(NamedTuple):
    id: str
    status: str
    grade: str
    maxgrade: str
    uniqueip: str


class CheckPointAsgChassisSgms(NamedTuple):
    id: str
    status: str


class CheckPointASGChassis(NamedTuple):
    info: CheckoointAsgChassisInfo
    chassis: List[CheckpointAsgChassisParams]
    sgms: List[CheckPointAsgChassisSgms]


def parse_checkpoint_asg_chassis(string_table: List[StringTable]) -> Optional[CheckPointASGChassis]:
    try:
        return CheckPointASGChassis(
            info=CheckoointAsgChassisInfo(*string_table[0][0]),
            chassis=[CheckpointAsgChassisParams(*chassis) for chassis in string_table[1]],
            sgms=[CheckPointAsgChassisSgms(*sgm) for sgm in string_table[2]]
        )
    except (TypeError, IndexError):
        pass


def discovery_checkpoint_asg_chassis(section: CheckPointASGChassis) -> DiscoveryResult:
    yield Service(parameters={
        'discovery_chassis': len(section.chassis),
        'discovery_sgms': len(section.sgms),
    })


def check_checkpoint_asg_chassis(params, section: CheckPointASGChassis) -> CheckResult:
    notice = ''
    notice += '\nTo verify this output use the "asg monitor" CLI command on the Check Point SMO,\n'
    notice += f'\nChassis Mode: {section.info.mode}'
    notice += f'\nH/A Mode: {section.info.hamode}'
    notice += f'\nSync to active: {section.info.synctoactive}'
    notice += f'\nSync to standby: {section.info.synctostandby}'

    yield Result(state=State.OK, notice=notice)

    for chassis in section.chassis:
        if chassis.uniqueip != 'N/A':
            yield Result(state=State.OK, notice=f'\nChassis {chassis.id} unique IP: {chassis.uniqueip}')

        if not chassis.status.lower() in ['active', 'standby']:
            yield Result(
                state=State(params['state_chassis_not_active_standby']),
                summary=f'Chassis {chassis.id}: {chassis.status}'
            )
        elif chassis.id == params['desired_chassis_active'] and chassis.status.lower() != 'active':
            yield Result(
                state=State(params['state_chassis_not_desired_active']),
                notice=f'Chassis {chassis.id}: {chassis.status} should be active'
            )
        else:
            yield Result(state=State.OK, summary=f'Chassis {chassis.id}: {chassis.status}')

        summary = f'grade: {chassis.grade}/{chassis.maxgrade}'
        details = f'Chassis {chassis.id} grade: {chassis.grade}/{chassis.maxgrade}'
        if chassis.grade != chassis.maxgrade:
            yield Result(state=State(params['state_chassis_not_max_grade']), summary=summary, details=details)
        else:
            yield Result(state=State.OK, summary=summary, details=details)

    if len(section.chassis) != params['discovery_chassis']:
        yield Result(
            state=State(params['state_chassis_number_changed']),
            notice=f'Number of chassis changed ({len(section.chassis)}/{params["discovery_chassis"]}'
        )

    active_sgms = 0
    for sgm in section.sgms:
        if sgm.status.lower() == 'active':
            active_sgms += 1
        else:
            yield Result(state=State(params['state_sgm_not_active']), notice=f'SGM {sgm.id} state: {sgm.status}')

    yield Result(state=State.OK, summary=f'{active_sgms}/{len(section.sgms)} SGMs active')

    if len(section.sgms) != params['discovery_sgms']:
        yield Result(
            state=State(params['state_sgms_number_changed']),
            notice=f'Number of SGMs changed ({len(section.sgms)}/{params["state_sgms_number_changed"]}'
        )


register.snmp_section(
    name='checkpoint_asg_chassis',
    parse_function=parse_checkpoint_asg_chassis,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.48.28',  # CHECKPOINT-MIB::asgChassisInfo
            oids=[
                '1',  # asgChassisMode
                '2',  # asgChassisHAMode
                '3.1',  # asgSyncToActive
                '3.2',  # asgSyncToStandby
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.48.28.4.1',  # CHECKPOINT-MIB::asgChassisParamsEntry
            oids=[
                '2',  # asgChassisParamsID
                '3',  # asgChassisParamsStatus
                '4',  # asgChassisParamsGrade
                '5',  # asgChassisParamsMaxGrade
                '6',  # asgChassisParamsUniqueIP
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.48.28.5.1',  # CHECKPOINT-MIB::asgSGMEntry
            oids=[
                '2',  # asgSGMID
                '3',  # asgSGMStatus
            ]
        ),
    ],
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_asg_chassis',
    service_name='ASG Chassis',
    discovery_function=discovery_checkpoint_asg_chassis,
    check_function=check_checkpoint_asg_chassis,
    check_default_parameters={
        'state_chassis_number_changed': 2,
        'desired_chassis_active': '1',
        'state_chassis_not_desired_active': 1,
        'state_chassis_not_active_standby': 2,
        'state_chassis_not_max_grade': 2,
        'state_sgm_not_active': 2,
        'state_sgms_number_changed': 2,
    },
    check_ruleset_name='checkpoint_asg_chassis',
)
